using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Syncfusion.Windows.Forms;
using Syncfusion.Calculate;
using System.Collections;


namespace ToSyncFusion_FormulaRetrieval
{
    public partial class Form1 : Form
    {
        IList<string> tabPageList;
        Syncfusion.Calculate.CalcEngine engine;
        public Form1()
        {
            InitializeComponent();
            tabPageList = new List<string>();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            CreateTabPage("Testing", GetMergedTable());

            CreateTabPage("Testing1", GetMergedTable());

            if (comboBoxAdvSheets.Items.Count > 0)
            {
                comboBoxAdvSheets.SelectedIndex = 0;
            }

        }

        private DataTable GetMergedTable()
        {
            DataTable dtPMAlign = CreatePMAlignDataTable();
            DataTable dtCogCaliper = CreateCogCliperDataTable();

            DataTable summaryTable = dtPMAlign.Copy();
            summaryTable.Merge(dtCogCaliper);
            return summaryTable;
        }
        private DataTable CreatePMAlignDataTable()
        {
            DataTable dtPMAlign = new DataTable("PMAlign");

            dtPMAlign.Columns.Add("ItemName");
            dtPMAlign.Columns.Add("Enabled", typeof(System.Boolean));
            dtPMAlign.Columns.Add("Units", typeof(Units));
            dtPMAlign.Columns.Add("Format", typeof(int));
            dtPMAlign.Columns.Add("GadgetName");
            dtPMAlign.Columns.Add("FormulaCell");
            dtPMAlign.Rows.Add(new object[] { "Score", "True", Units.Linear, 1, "PMAlign" });
            dtPMAlign.Rows.Add(new object[] { "PosX", "True", Units.NonLinear, 2, "PMAlign" });
            return dtPMAlign;

        }

        enum Units
        {
            mm,
            m,
            deg,
            Linear,
            NonLinear,
        }

        private DataTable CreateCogCliperDataTable()
        {

            DataTable dtCogCaliper = new DataTable("CogCaliper");

            dtCogCaliper.Columns.Add("ItemName");
            dtCogCaliper.Columns.Add("Enabled", typeof(System.Boolean));
            dtCogCaliper.Columns.Add("Units", typeof(Units));
            dtCogCaliper.Columns.Add("Format", typeof(int));
            dtCogCaliper.Columns.Add("GadgetName");
            dtCogCaliper.Columns.Add("FormulaCell");
            dtCogCaliper.Rows.Add(new object[] { "StartTime", "True", Units.Linear, 3, "CogCaliper" });
            dtCogCaliper.Rows.Add(new object[] { "EndTime", "True", Units.NonLinear, 4, "CogCaliper" });
            return dtCogCaliper;
        }



        private void CreateTabPage(string pageName, DataTable dt)
        {
            if (!tabPageList.Contains(pageName))
            {
                TabBarPage tabPage = new Syncfusion.Windows.Forms.TabBarPage(pageName);
                tabPage.SuspendLayout();
                tabPage.Controls.Add(CreateDataGrid(pageName, dt));
                tabPage.Location = new System.Drawing.Point(0, 0);
                tabPage.Name = pageName;
                tabPage.SplitBars = Syncfusion.Windows.Forms.DynamicSplitBars.None;
                tabPage.TabBackColor = System.Drawing.SystemColors.Control;
                tabPage.ThemesEnabled = false;
                tabPage.ResumeLayout(false);
                AddTabPageToControl(tabPage);
                comboBoxAdvSheets.Items.Add(tabPage.Name);

            }
            else
            {
                CalcGridDataBoundGrid calcGridDataBoundGrid = GetGridByName(pageName);
                if (calcGridDataBoundGrid != null)
                {
                    calcGridDataBoundGrid.DataSource = dt;
                }
                
                SetUpGridFormat(calcGridDataBoundGrid);
            }

        }


        private CalcGridDataBoundGrid CreateDataGrid(string gridName, DataTable dt)
        {
            CalcGridDataBoundGrid calcGridDataBoundGrid = new CalcGridDataBoundGrid();

            //calcGridDataBoundGrid.CellClick += new Syncfusion.Windows.Forms.Grid.GridCellClickEventHandler(DataGridCellClick);
            //calcGridDataBoundGrid.ContextMenuStrip = contextMenuGrid;

            calcGridDataBoundGrid.Name = gridName;

            if (dt != null)
            {
                calcGridDataBoundGrid.DataSource = dt;
            }


            SetUpGridFormat(calcGridDataBoundGrid);
           
            return calcGridDataBoundGrid;
        }


        private CalcGridDataBoundGrid GetGridByName(string pageName)
        {
            CalcGridDataBoundGrid calcGridDataBoundGrid = null;

            TabBarPage tabPage = (TabBarPage)tabBarSplitterControl1.Controls[pageName];
            if (tabPage != null && tabPage.Controls.Count > 0)
            {
                calcGridDataBoundGrid = (CalcGridDataBoundGrid)tabPage.Controls[pageName];
            }

            return calcGridDataBoundGrid;
        }


        private void AddTabPageToControl(TabBarPage tabPage)
        {
            tabBarSplitterControl1.Controls.Add(tabPage);

            if (!tabPageList.Contains(tabPage.Name))
            {
                tabPageList.Add(tabPage.Name);
            }
        }
        
        int sheetfamilyID = int.MinValue;
        private void SetUpGridFormat(CalcGridDataBoundGrid grid)
        {
            grid.GridVisualStyles = Syncfusion.Windows.Forms.GridVisualStyles.Office2007Blue;
            grid.Properties.BackgroundColor = System.Drawing.Color.FromArgb(((System.Byte)(227)), ((System.Byte)(239)), ((System.Byte)(255)));
            grid.Properties.GridLineColor = System.Drawing.Color.FromArgb(((System.Byte)(208)), ((System.Byte)(215)), ((System.Byte)(229)));
            grid.Model.Options.DefaultGridBorderStyle = Syncfusion.Windows.Forms.Grid.GridBorderStyle .Solid;
            grid.ForeColor = System.Drawing.Color.MidnightBlue;
            grid.Font = new System.Drawing.Font("Verdana", 8.5F);


            if (engine == null)
            {
                //Create the engine:
                engine = new Syncfusion.Calculate.CalcEngine(grid);
            }
            //Track dependencies required for auto calculations:
            engine.UseDependencies = true;

            //Register multiple ICalcData objects for cross sheet references:
            if (sheetfamilyID == int.MinValue)
            {
                sheetfamilyID = Syncfusion.Calculate.CalcEngine.CreateSheetFamilyID();
            }
            engine.RegisterGridAsSheet(grid.Name, grid, sheetfamilyID);


            //engine.FormulaParsing += new Syncfusion.Calculate.FormulaParsingEventHandler(engine_FormulaParsing);

        }

        private Hashtable parentObjectToSheetName = new Hashtable();

        private void buttonAdvgetFormulaCells_Click(object sender, EventArgs e)
        {
            string selectedSheetName = comboBoxAdvSheets.SelectedItem.ToString();
            CalcGridDataBoundGrid calcGridDataBoundGrid = GetGridByName(comboBoxAdvSheets.SelectedItem.ToString());

            if (calcGridDataBoundGrid != null)
            {
                Syncfusion.Calculate.GridSheetFamilyItem family = Syncfusion.Calculate.CalcEngine.GetSheetFamilyItem(calcGridDataBoundGrid);


                foreach (object key in engine.FormulaInfoTable.Keys)
                {
                    FormulaInfo info = engine.FormulaInfoTable[key] as FormulaInfo;
                    string cell = key.ToString();
                    int loc = cell.LastIndexOf('!');
                    string sheetToken = cell.Substring(0, loc + 1);
                    CalcGridDataBoundGrid calcGrid = family.TokenToParentObject[sheetToken] as CalcGridDataBoundGrid;
                    if (calcGrid != null && selectedSheetName == calcGrid.Name)
                    {
                        cell = cell.Substring(loc + 1);
                        Console.WriteLine("sheet=[{0}] cell=[{1}] formula=[{2}]", selectedSheetName, cell, info.FormulaText);
                    }
                }


                //Can you please expose this private function(sheetFormulaInfoTable) so that i can get the formulas for the selected grid.
                
                //After so many emails going back and forth i was not able to get a working solution 
                //for my requirement.

                //OR 

                // Is there any other way i can get the formulas for the selected grid.
                



                //family.sheetFormulaInfoTable
                
            }


        }
    }
}